#!/usr/bin/perl -w
#
# ASP auditor v1.0 BETA
#
# Author david.kierznowski_at_gmail.com
# http://michaeldaw.org 
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#

use Socket;
use strict;

my $progver = "ASP Audit v1.0 (BETA)";
my $author = "david.kierznowski\@gmail.com";
my $new_rand = int(rand(10000));

use strict;

    use vars qw/ %opt /;

    sub init()
    {
        use Getopt::Std;
        my $opt_string = 'hvpbm';
        getopts( "$opt_string", \%opt ) or usage();
        usage() if $opt{h};
    }

init();

    sub usage()
    {
        print "$progver [ $author ]
        Usage:   $0 (opts) [host] [port]

        (opts)
            -h these usage instructions
            -b brute force ASP.NET version using JS Validate
            directories.
            -m match against fingerprints 
            -v verbose messaging\n\n";

        exit(1);
    }

my $verbose = 1 if $opt{v};
my $bf = 1 if $opt{b};
my $mv = 1 if $opt{m};

 my $ipaddr = shift or die usage();
 my $port = 80;
 my ($rdata, $sendvar, @data);
 my $aspdir = "aspnet_client/system_web/";
 my $asp = 0;
 my $aspver = "Unknown";
 my $aspver2 = "Unknown";

 my @aspver=
 ("1_0_2204",
 "1_0_2914",
 "1_0_3512",
 "1_0_3705",
 "1_1_4322",
 "1_2_21213",
 "1_2_30703",
 "2_0_40301",
 "2_0_40426",
 "2_0_40607",
 "2_0_40903",
 "2_0_41115",
 "2_0_50110",
 "2_0_50215",
 "2_0_50601",
 "2_0_50727");

sub connect_me {

   my $proto = getprotobyname('tcp') or die "getprotobyname";
   socket(conn_new, AF_INET, SOCK_STREAM, $proto) or die "socket()";

   my $sin = sockaddr_in($port,inet_aton($ipaddr)) or die "socket()";
   connect(conn_new,$sin) or die "Unable to reach host/port\n";

   print "Connected to $ipaddr:[$port]\n" if $verbose;
   my $firefox_hdr = "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1;en-US;rv:1.8.0.6) Gecko/20060728 Firefox/1.5.0.6\r\nAccept:text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5\r\nAccept-Language: en-us,en;q=0.5\r\nAccept-Encoding:gzip,deflate\r\nAccept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7\r\nKeep-Alive: 300\r\nConnection:keep-alive\r\n\r\n";
  
   print "Sending: $sendvar\n" if $verbose;
   my $senddata = "GET /$sendvar HTTP/1.1\r\nHost: $ipaddr\r\n$firefox_hdr\r\n\r\nConnection: Close\r\n\r\n";
                 
   send (conn_new, $senddata, 0); 
   
    do { 
     $rdata = <conn_new>;
     push (@data, $rdata);
    } until ($rdata =~ /\<\/html\>/i); 

      print @data if ($verbose);
      close (conn_new);

   print @data if $verbose;
}


sub check_aspver() {
$sendvar = "$new_rand.aspx?<script>$new_rand</script>"; 
connect_me();

my $aspfound = "No";
my $svr_hdr = "Unknown";
my $asp_val = "No";
my $asp_dbg = "No";
my $asp_verbose = "No";

foreach my $line (@data) {
 
  if ($line =~ /X-Powered-By: ASP.NET/) {
   $aspfound = "YES"; 
  }

  if ($line =~ /Server: (.*)\W/) {
   $svr_hdr = $1;
  }

  if ($line =~ /: (\d\.\d\.\d+)/) {
   $aspver = $1;
   $asp = 1;
  }

  if ($line =~ /(\d\.\d\.\d+\.\d+)/i) {
   $aspver2 = $1;
   $asp = 2;
  }
 
  if ($line =~ /for security reasons/i) {
   $asp_val = "YES";
  }
 
  if ($line =~ /Server Error/i) {
   $asp_dbg = "YES";
  }

  if ($line =~ /ASP.NET is configured to show verbose/i) {
   $asp_verbose = "YES";
  }

}
 
  print "Target: $ipaddr \n";
  print "Server Software: " . $svr_hdr . "\n";
  print "ASP Framework: $aspfound \n";
  print "ASP Simple Version: $aspver \n";
  print "ASP Specific Version: $aspver2 \n";
  print "ASP verbose messages: $asp_verbose \n";

  if ($asp_dbg =~ /Enabled/) { 
   print "ASP Validate: Disabled \n";
  } else {
   print "ASP Validate: $asp_val \n"; 
  }
  print "Default Error Messages: $asp_dbg \n";
  print "\n";

 if ($verbose) {
  if ($aspfound =~ /YES/ && $asp_dbg !~ /YES/ && $asp_val !~ /YES/) {
   print "Summary: The .NET framework looks fairly hardened.\n\n";
  }
 }

}

sub check_aspdir() {
 foreach my $line (@aspver) {
  @data = 0;
  $sendvar = $aspdir . $line;

  print "\nTrying: $sendvar\n" if $verbose;
  connect_me();
  parse();
 }

}

sub parse() {

   foreach my $html (@data) {
    if ($html =~ /Listing Denied\<\/title\>/) {
    print "Found: $sendvar\n";
    }
   }
 }

sub get_version() {

 if ($asp == 2) {
  my $version = `grep $aspver2 asp-ver.txt`;
  print "Fingerprint matches: \n";
  print $version;
 } 
 if ($asp == 1) {
  print "Fingerprint matches: \n";
  my $version = `grep $aspver asp-ver.txt`;
  print $version;
 }

}


#### main()

check_aspver();
check_aspdir() if $bf;
get_version() if $mv;
